library(R2jags);library(snowfall);library(survival);library(msm)
setwd("C:/Users/TMurray/Documents/Thesis") #Set Your Working Directory
#Create a folder in the working directory called Results, the simulation
#results will be written to .txt files in this folder if you uncomment an
#annotated section of the code in the simfun() below.

#JAGS Code for the Piecewise Exponential Reference Models (Current/Pooled PE)
write("data{
	  for (i in 1:n) {
	         for (k in 1:K) {
	            # indicates event-time in interval k
			zed[i,k] <- delta[i]*step(y[i] - kappa[k] - 0.00001)*step(kappa[k+1] - y[i]);   
				
		      # length of overlap of y[i] with interval k
		      theta[i,k] <- (min(y[i], kappa[k+1]) - kappa[k])*step(y[i] - kappa[k]);      
	         }
	   } 
}


model{ 
	  for (i in 1:n) {
	  	#Time Axis Partition is defined in R code:
	         for (k in 1:K) {
		    # define the likelihood
			zed[i,k] ~ dpois(mu[i,k]);
			mu[i,k] <- theta[i,k]*exp(alpha[k]);
	         }
	   } 


	#Random Walk Prior Process on Log-Hazards
	  alpha[1] ~ dnorm(0.0, 0.0001);
	  for (k in 2:K) {
	      alpha[k] ~ dnorm(alpha[k-1], eta);
	  } 
	   eta <- 1/(sigma*sigma);
	   sigma ~ dunif(0.01,100);
}","PERefMod.txt")


#JAGS Code for Our Model with no covariate adjustments (Comm PE)
write("data{
 ## Data Construction for the Historical Likelihood ##
	  for (i0 in 1:n0) {
	         for (k in 1:K) {
	            # indicates event-time in interval k
			zed0[i0,k] <- delta0[i0]*step(y0[i0] - kappa[k] - 0.00001)*step(kappa[k+1] - y0[i0]);   
				
		      # length of overlap of y0[i0] with interval k
		      theta0[i0,k] <- (min(y0[i0], kappa[k+1]) - kappa[k])*step(y0[i0] - kappa[k]);      
			
	         }
	   } 
 ## Data Construction for the Current Likelihood ##
	  for (i in 1:n) {
	         for (k in 1:K) {
	            # indicates event-time in interval k
			zed[i,k] <- delta[i]*step(y[i] - kappa[k] - 0.00001)*step(kappa[k+1] - y[i]);   
				
		      # length of overlap of y[i] with interval k
		      theta[i,k] <- (min(y[i], kappa[k+1]) - kappa[k])*step(y[i] - kappa[k]);      
	         }
	   } 
}

model{ 
 ## Historical Likelihoods ##
	  for (i0 in 1:n0) {
	         for (k in 1:K) {
		    # define the likelihood
			zed0[i0,k] ~ dpois(mu0[i0,k]);
			mu0[i0,k] <- theta0[i0,k]*exp(alpha0[k]);
	         }
	   } 

 ## Current Likelihoods ##
	  for (i in 1:n) {
	         for (k in 1:K) {			
		    # define the likelihood
			zed[i,k] ~ dpois(mu[i,k]);
			mu[i,k] <- theta[i,k]*exp(alpha[k]);
	         }
	   } 

	#Vague Priors for Treatment Effect and Historical Log-Hazards
	#Commensurate Priors on Current Log-Hazards
	  	 alpha0[1] ~ dnorm(0,0.0001);
	  	 alpha[1] ~ dnorm(alpha0[1], tau);
		 v <- tau/spike;
	  for (k in 2:K) {
		 alpha0[k] ~ dnorm(alpha0[k-1], eta0);
		 alpha[k] ~ dnorm(v*alpha0[k]+(1-v)*alpha[k-1], v*tau + (1-v)*eta);
	  } 
		 eta0 <- 1/(sigma0*sigma0);
		 eta <- 1/(sigma*sigma);
		 sigma0 ~ dunif(0.01,100);
		 sigma ~ dunif(0.01,100);
	#Defining Spike and Slab Prior;
		 t ~ dbern(p0);
		 slab ~ dunif(l.slab,u.slab);
		 tau <- (1-t)*spike + t*slab;
}","CommMod.txt")

#############################
#   Arguments to simfun()   #
#############################
#sim.type = c('Exp', 'Weib'), Simulate Exponential or Weibull data as described in the paper
#sp.sl.pars takes the arguments, c(S_l, S_u, R, p_0) which define the "Spike and Slab" prior placed on tau
#grid.pts, # of grid points used to calculate IME and IMSE
#pw.time, follow-up time at which to calculate interval width and coverage probabilities, must be in (0,1] and is .75 in the paper
#pw.alpha, 100(1-pw.alpha)% confidence/credible intervals are used when calculated interval width and coverage probabilities, set to .05 in the paper

simfun = function(sim.type='Exp',sp.sl.pars=c(0.0001,2,200,0.9), grid.pts=2000, pw.time=0.75, pw.alpha=.05){
if(sim.type=='Exp'){
#Simulate Current Dataset
	curr.rate = sample(exp(c(runif(1,-log(4),log(4)),-log(4),-log(3),-log(2),-log(1.5),log(1),log(1.5),log(2),log(3),log(4))),size=1,prob=c(.5,rep(.5/9,9)))
	#curr.rate = sample(exp(c(runif(1,log(.5),log(2)),log(.5),log(.75),log(1),log(1.5),log(2))),size=1,prob=c(.5,.1,.1,.1,.1,.1))
	n=80;t=rexp(n,curr.rate);c=runif(n,0,2);yy.=unlist(lapply(1:n,function(x) min(t[x],c[x])));y.=sort(yy.)
	dd.=unlist(lapply(1:n, function(x){ ifelse(t[x]==min(t[x],c[x]),1,0)} ));d.=dd.[order(yy.)]
	y=ifelse(y.<1,y.,1);delta=ifelse(y.<1 & d.==1,1,0)

#Simulate Historical Dataset
	n0=200;t0=rexp(n0);c0=runif(n0,0,2);yy0.=unlist(lapply(1:n0,function(x) min(t0[x],c0[x])));y0.=sort(yy0.)
	dd0.=unlist(lapply(1:n0, function(x){ ifelse(t0[x]==min(t0[x],c0[x]),1,0)} ));d0.=dd0.[order(yy0.)]
	y0=ifelse(y0.<1,y0.,1);delta0=ifelse(y0.<1 & d0.==1,1,0)
 }
if(sim.type=='Weib'){
#Simulate Current Dataset
	curr.shape = sample(exp(c(runif(1,-log(4),log(4)),-log(4),-log(3),-log(2),-log(1.5),log(1),log(1.5),log(2),log(3),log(4))),size=1,prob=c(.5,rep(.5/9,9)))
	#curr.shape = sample(exp(c(runif(1,log(.5),log(2)),log(.5),log(.75),log(1),log(1.5),log(2))),size=1,prob=c(.5,.1,.1,.1,.1,.1))
	n=80;t=rweibull(n,shape=curr.shape,scale=1);c=runif(n,0,2);yy.=unlist(lapply(1:n,function(x) min(t[x],c[x])));y.=sort(yy.)
	dd.=unlist(lapply(1:n, function(x){ ifelse(t[x]==min(t[x],c[x]),1,0)} ));d.=dd.[order(yy.)]
	y=ifelse(y.<1,y.,1);delta=ifelse(y.<1 & d.==1,1,0)

#Simulate Historical Dataset
	n0=200;t0=rweibull(n0,shape=1,scale=1);c0=runif(n0,0,2);yy0.=unlist(lapply(1:n0,function(x) min(t0[x],c0[x])));y0.=sort(yy0.)
	dd0.=unlist(lapply(1:n0, function(x){ ifelse(t0[x]==min(t0[x],c0[x]),1,0)} ));d0.=dd0.[order(yy0.)]
	y0=ifelse(y0.<1,y0.,1);delta0=ifelse(y0.<1 & d0.==1,1,0)
 }


#Calculating Knot Placements
K=max(5,min(floor(sum(delta)/8),20))
kappa=c(0,quantile(y[delta==1],seq(0,1,by=1/K)[-c(1,K+1)]),1)

#Fit Current Reference Models (Piecewise Exponential and Kaplan-Meier)
dta <- list("n", "K", "kappa", "y", "delta")
ints <- function() {list(sigma=runif(1,0.01,100), alpha=runif(K,0,0))}
pars <- c("alpha")
curr.fit <- jags(data=dta,inits=ints,model.file="PERefMod.txt",parameters=pars,
n.chains=1,n.iter=20200,n.burnin=200,n.thin=1)$BUGSoutput

#Current K-M
curr.km = survfit(Surv(y,delta)~1, conf.int=1-pw.alpha, conf.type='log-log')

#Fit Commensurate Model
#Spike and Slab Specification
l.slab = sp.sl.pars[1]; u.slab=sp.sl.pars[2]; spike=sp.sl.pars[3]; p0=sp.sl.pars[4]
dta <- list("n0", "n", "K", "kappa", "y0", "delta0", "y", "delta", "l.slab", "u.slab", "spike", "p0")
ints <- function() {list(sigma0=runif(1,0.01,100), sigma=runif(1,0.01,100), alpha0=runif(K,0,0), alpha=runif(K,0,0))}
comm.fit <- jags(data=dta,inits=ints,model.file="CommMod.txt",parameters=pars,
n.chains=1,n.iter=22000,n.burnin=2000,n.thin=1)$BUGSoutput

#Fit Pooled Reference Model (Piecewise Exponential and Kaplan-Meier)
y..=c(y,y0);d..=c(delta,delta0)
n=n+n0;y=sort(y..);delta=d..[order(y..)]
dta <- list("n", "K", "kappa", "y", "delta")
ints <- function() {list(sigma=runif(1,0.01,100), alpha=runif(K,0,0))}
pool.fit <- jags(data=dta,inits=ints,model.file="PERefMod.txt",parameters=pars,
n.chains=1,n.iter=20200,n.burnin=200,n.thin=1)$BUGSoutput

#Fit Pooled K-M
pool.km = survfit(Surv(y,delta)~1, conf.int=1-pw.alpha, conf.type='log-log')

#Calculate Integrated Error, Integrated Squared Error, and Coverage Indicator
#First Compute Survival Curve Across Grid of Points
#For PE Models
PwExpSurv<-function(fit,grid.pts){
  t.grid<-round(c(kappa[length(kappa)]*(1:grid.pts)/grid.pts),5)
  s.hat = ppexp(t.grid, rate=exp(fit$mean$alpha), t=as.vector(kappa[-(K+1)]), lower.tail = FALSE)
  return(list(t.grid=t.grid,s.hat=s.hat))
}
#For K-M Models
KMSurv<-function(fit,grid.pts){
  t.grid<-round(c(kappa[length(kappa)]*(1:grid.pts)/grid.pts),5)
  s.hat<-sapply(t.grid,function(t.grid){return(ifelse(sum(ifelse(t.grid>fit$time,1,0))>1,fit$surv[sum(ifelse(t.grid>fit$time,1,0))],1))})
  return(list(t.grid=t.grid,s.hat=s.hat))
}

#Function to Compute Width and Coverage of 95% Credible Interval for Survival at End of Follow-up
PwExpCI<-function(fit,type='Exp',time=0.75,alpha=0.05){
  kk = sum(kappa<time)
  q = rbind(c(c(kappa[-1]-kappa[-(K+1)])[-(kk:K)],time-kappa[kk]))
  qq=t(exp(fit$sims.matrix[,1:kk]))
  ci = quantile(exp(-q%*%qq),c(alpha/2,1-alpha/2)); wth = diff(ci) 
  if(type=='Exp')
	cvr = ifelse(ci[1]<exp(-time*curr.rate) & ci[2]>exp(-time*curr.rate),1,0)
  if(type=='Weib')
	cvr = ifelse(ci[1]<pweibull(time,shape=curr.shape,scale=1,lower.tail=FALSE) & ci[2]>pweibull(time,shape=curr.shape,scale=1,lower.tail=FALSE),1,0)
  if(type=='Odd')
	cvr = ifelse(ci[1]<ifelse(time<=.25,pweibull(time,shape=.5,scale=1,lower.tail=FALSE),pweibull(.25,shape=.5,scale=1,lower.tail=FALSE)*exp(-(time-.25)*curr.rate)) & ci[2]>ifelse(time<=.25,pweibull(time,shape=.5,scale=1,lower.tail=FALSE),pweibull(.25,shape=.5,scale=1,lower.tail=FALSE)*exp(-(time-.25)*curr.rate)),1,0)
  return(cbind(wth,cvr))
}

#Function to Compute Width and Coverage of 95% Confidence Interval for Survival at End of Follow-up
KMCI<-function(fit,type='Exp',time=0.75){
  kk = sum(fit$time<time)
  ci = c(fit$lower[kk],fit$upper[kk]); wth = diff(ci)
  if(type=='Exp')
  	cvr = ifelse(ci[1]<exp(-time*curr.rate)&ci[2]>exp(-time*curr.rate),1,0)
  if(type=='Weib')
  	cvr = ifelse(ci[1]<pweibull(time,shape=curr.shape,scale=1,lower.tail=FALSE) & ci[2]>pweibull(time,shape=curr.shape,scale=1,lower.tail=FALSE),1,0)
  if(type=='Odd')
	cvr = ifelse(ci[1]<ifelse(time<=.25,pweibull(time,shape=.5,scale=1,lower.tail=FALSE),pweibull(.25,shape=.5,scale=1,lower.tail=FALSE)*exp(-(time-.25)*curr.rate)) & ci[2]>ifelse(time<=.25,pweibull(time,shape=.5,scale=1,lower.tail=FALSE),pweibull(.25,shape=.5,scale=1,lower.tail=FALSE)*exp(-(time-.25)*curr.rate)),1,0)
  return(cbind(wth,cvr))
}

#Run Functions
curr.surv <- PwExpSurv(fit=curr.fit,grid.pts=grid.pts)
curr.covr <- PwExpCI(fit=curr.fit,type=sim.type,time=pw.time,alpha=pw.alpha)
curr.km.surv <- KMSurv(fit=curr.km,grid.pts=grid.pts)
curr.km.covr <- KMCI(fit=curr.km,type=sim.type,time=pw.time)
comm.surv <- PwExpSurv(fit=comm.fit,grid.pts=grid.pts)
comm.covr <- PwExpCI(fit=comm.fit,type=sim.type,time=pw.time,alpha=pw.alpha)
pool.surv <- PwExpSurv(fit=pool.fit,grid.pts=grid.pts)
pool.covr <- PwExpCI(fit=pool.fit,type=sim.type,time=pw.time,alpha=pw.alpha)
pool.km.surv <- KMSurv(fit=pool.km,grid.pts=grid.pts)
pool.km.covr <- KMCI(fit=pool.km,type=sim.type,time=pw.time)

if(sim.type=='Exp'){
	#Calculating IE/ISE
	curr.ie<-sum((curr.surv$t.grid[2]-curr.surv$t.grid[1])*(curr.surv$s.hat-exp(-curr.surv$t.grid*curr.rate)))
	curr.ise<-sum((curr.surv$t.grid[2]-curr.surv$t.grid[1])*(curr.surv$s.hat-exp(-curr.surv$t.grid*curr.rate))**2)
	curr.km.ie<-sum((curr.km.surv$t.grid[2]-curr.km.surv$t.grid[1])*(unlist(curr.km.surv$s.hat)-exp(-curr.km.surv$t.grid*curr.rate)))
	curr.km.ise<-sum((curr.km.surv$t.grid[2]-curr.km.surv$t.grid[1])*(unlist(curr.km.surv$s.hat)-exp(-curr.km.surv$t.grid*curr.rate))**2)
	comm.ie<-sum((comm.surv$t.grid[2]-comm.surv$t.grid[1])*(comm.surv$s.hat-exp(-comm.surv$t.grid*curr.rate)))
	comm.ise<-sum((comm.surv$t.grid[2]-comm.surv$t.grid[1])*(comm.surv$s.hat-exp(-comm.surv$t.grid*curr.rate))**2)
	pool.ie<-sum((pool.surv$t.grid[2]-pool.surv$t.grid[1])*(pool.surv$s.hat-exp(-pool.surv$t.grid*curr.rate)))
	pool.ise<-sum((pool.surv$t.grid[2]-pool.surv$t.grid[1])*(pool.surv$s.hat-exp(-pool.surv$t.grid*curr.rate))**2)
	pool.km.ie<-sum((pool.km.surv$t.grid[2]-pool.km.surv$t.grid[1])*(unlist(pool.km.surv$s.hat)-exp(-pool.km.surv$t.grid*curr.rate)))
	pool.km.ise<-sum((pool.km.surv$t.grid[2]-pool.km.surv$t.grid[1])*(unlist(pool.km.surv$s.hat)-exp(-pool.km.surv$t.grid*curr.rate))**2)

#Write Results to .txt files (Uncomment if you want to write out results to file)
	#write.table(cbind(curr.rate,curr.ie,curr.ise,curr.covr),"Results/CurrExpRes.txt",row.names=FALSE,col.names=FALSE,append=TRUE,sep=",")
	#write.table(cbind(curr.rate,curr.km.ie,curr.km.ise,curr.km.covr),"Results/CurrKMExpRes.txt",row.names=FALSE,col.names=FALSE,append=TRUE,sep=",")
	#write.table(cbind(curr.rate,comm.ie,comm.ise,comm.covr),paste("Results/CommExpResSp",spike,"p0",p0,"su",u.slab,".txt",sep=""),row.names=FALSE,col.names=FALSE,append=TRUE,sep=",")
	#write.table(cbind(curr.rate,pool.ie,pool.ise,pool.covr),"Results/PoolExpRes.txt",row.names=FALSE,col.names=FALSE,append=TRUE,sep=",")
	#write.table(cbind(curr.rate,pool.km.ie,pool.km.ise,pool.km.covr),"Results/PoolKMExpRes.txt",row.names=FALSE,col.names=FALSE,append=TRUE,sep=",")
#Return Results
	return(cbind(curr.rate,curr.ie,curr.ise,curr.covr,comm.ie,comm.ise,comm.covr,pool.ie,pool.ise,pool.covr,curr.km.ie,curr.km.ise,curr.km.covr,pool.km.ie,pool.km.ise,pool.km.covr))
 }
if(sim.type=='Weib'){
	#Calculating IE/ISE
	curr.ie<-sum((curr.surv$t.grid[2]-curr.surv$t.grid[1])*(curr.surv$s.hat-pweibull(curr.surv$t.grid,shape=curr.shape,scale=1,lower.tail=FALSE)))
	curr.ise<-sum((curr.surv$t.grid[2]-curr.surv$t.grid[1])*(curr.surv$s.hat-pweibull(curr.surv$t.grid,shape=curr.shape,scale=1,lower.tail=FALSE))**2)
	curr.km.ie<-sum((curr.km.surv$t.grid[2]-curr.km.surv$t.grid[1])*(unlist(curr.km.surv$s.hat)-pweibull(curr.surv$t.grid,shape=curr.shape,scale=1,lower.tail=FALSE)))
	curr.km.ise<-sum((curr.km.surv$t.grid[2]-curr.km.surv$t.grid[1])*(unlist(curr.km.surv$s.hat)-pweibull(curr.surv$t.grid,shape=curr.shape,scale=1,lower.tail=FALSE))**2)
	comm.ie<-sum((comm.surv$t.grid[2]-comm.surv$t.grid[1])*(comm.surv$s.hat-pweibull(curr.surv$t.grid,shape=curr.shape,scale=1,lower.tail=FALSE)))
	comm.ise<-sum((comm.surv$t.grid[2]-comm.surv$t.grid[1])*(comm.surv$s.hat-pweibull(curr.surv$t.grid,shape=curr.shape,scale=1,lower.tail=FALSE))**2)
	pool.ie<-sum((pool.surv$t.grid[2]-pool.surv$t.grid[1])*(pool.surv$s.hat-pweibull(curr.surv$t.grid,shape=curr.shape,scale=1,lower.tail=FALSE)))
	pool.ise<-sum((pool.surv$t.grid[2]-pool.surv$t.grid[1])*(pool.surv$s.hat-pweibull(curr.surv$t.grid,shape=curr.shape,scale=1,lower.tail=FALSE))**2)
	pool.km.ie<-sum((pool.km.surv$t.grid[2]-pool.km.surv$t.grid[1])*(unlist(pool.km.surv$s.hat)-pweibull(curr.surv$t.grid,shape=curr.shape,scale=1,lower.tail=FALSE)))
	pool.km.ise<-sum((pool.km.surv$t.grid[2]-pool.km.surv$t.grid[1])*(unlist(pool.km.surv$s.hat)-pweibull(curr.surv$t.grid,shape=curr.shape,scale=1,lower.tail=FALSE))**2)

#Write Results to .txt files (Uncomment if you want to write out results to file)
	#write.table(cbind(curr.shape,curr.ie,curr.ise,curr.covr),"Results/CurrWeibRes.txt",row.names=FALSE,col.names=FALSE,append=TRUE,sep=",")
	#write.table(cbind(curr.shape,curr.km.ie,curr.km.ise,curr.km.covr),"Results/CurrKMWeibRes.txt",row.names=FALSE,col.names=FALSE,append=TRUE,sep=",")
	#write.table(cbind(curr.shape,comm.ie,comm.ise,comm.covr),paste("Results/CommWeibResSp",spike,"p0",p0,"su",u.slab,".txt",sep=""),row.names=FALSE,col.names=FALSE,append=TRUE,sep=",")
	#write.table(cbind(curr.shape,pool.ie,pool.ise,pool.covr),"Results/PoolWeibRes.txt",row.names=FALSE,col.names=FALSE,append=TRUE,sep=",")
	#write.table(cbind(curr.shape,pool.km.ie,pool.km.ise,pool.km.covr),"Results/PoolKMWeibRes.txt",row.names=FALSE,col.names=FALSE,append=TRUE,sep=",")
#Return Results
	return(cbind(curr.shape,curr.ie,curr.ise,curr.covr,comm.ie,comm.ise,comm.covr,pool.ie,pool.ise,pool.covr,curr.km.ie,curr.km.ise,curr.km.covr,pool.km.ie,pool.km.ise,pool.km.covr))
 }
} #End Simulation Function


#############################
# Run Simulations Parallely #
#############################
sfInit(socketHosts=rep("localhost",8),cpus=8,type='SOCK',parallel=TRUE)
# calling user written functions
sfExport('simfun')
# loading libraries in nodes
sfLibrary(MASS);sfLibrary(R2jags);sfLibrary(survival);sfLibrary(msm)
# run exponential simulation
res.exp = sfSapply(1:2000,function(x) simfun(sim.type='Exp',sp.sl.pars=c(0.0001,2,500,0.99)))
# run Weibull simulation
res.weib = sfSapply(1:2000,function(x) simfun(sim.type='Weib',sp.sl.pars=c(0.0001,2,200,0.9)))
# close nodes
sfStop()

#############################
#      Compile Results      #
#############################
res = res.exp
#res = res.weib
alpha = res[1,] # alpha's

#Current PE, Commensurate PE, Pooled PE, Current K-M, Pooled K-M
ies = res[c(2,6,10,14,18),] #IE
ises = res[c(2,6,10,14,18)+1,] #ISE
ciwds = res[c(2,6,10,14,18)+2,] #CI Width
cvrs = res[c(2,6,10,14,18)+3,] #Covers True Value

##or read in results files
#sp.sl.pars = c(.0001,2,200,.9)
#l.slab=sp.sl.pars[1];u.slab=sp.sl.pars[2];spike=sp.sl.pars[3];p0=sp.sl.pars[4];
#res = t(cbind(read.table("Results/CurrExpRes.txt",header=FALSE,sep=","),read.table(paste("Results/CommExpResSp",spike,"p0",p0,"su",u.slab,".txt",sep=""),header=FALSE,sep=",")[,-1],read.table("Results/PoolExpRes.txt",header=FALSE,sep=",")[,-1],read.table("Results/CurrKMExpRes.txt",header=FALSE,sep=",")[,-1],read.table("Results/PoolKMExpRes.txt",header=FALSE,sep=",")[,-1]))
##res = t(cbind(read.table("Results/CurrWeibRes.txt",header=FALSE,sep=","),read.table(paste("Results/CommWeibResSp",spike,"p0",p0,"su",u.slab,".txt",sep=""),header=FALSE,sep=",")[,-1],read.table("Results/PoolWeibRes.txt",header=FALSE,sep=",")[,-1],read.table("Results/CurrKMWeibRes.txt",header=FALSE,sep=",")[,-1],read.table("Results/PoolKMWeibRes.txt",header=FALSE,sep=",")[,-1]))
##res = t(cbind(read.table("Results/CurrOddRes.txt",header=FALSE,sep=","),read.table(paste("Results/CommOddResSp",spike,"p0",p0,"su",u.slab,".txt",sep=""),header=FALSE,sep=",")[,-1],read.table("Results/PoolOddRes.txt",header=FALSE,sep=",")[,-1],read.table("Results/CurrKMOddRes.txt",header=FALSE,sep=",")[,-1],read.table("Results/PoolKMOddRes.txt",header=FALSE,sep=",")[,-1]))
#alphas = res[1,] # alphas
##Current PE, Commensurate PE, Pooled PE, Current K-M, Pooled K-M
#ies = res[c(2,6,10,14,18),] #IE
#ises = res[c(2,6,10,14,18)+1,] #ISE
#ciwds = res[c(2,6,10,14,18)+2,] #CI Width
#cvrs = res[c(2,6,10,14,18)+3,] #Covers True Value

#############################
#      Look at Results      #
#############################
library(mgcv)
#############################
#          IME Plot         #
#############################
#Trim off excess margin space (bottom, left, top, right)
par(mar=c(4.2, 4.2, 0.2, 0.2))
#Curr PE
fit.ie1=gam(ies[1,]~s(log(alphas),bs='cr',k=20),family=gaussian)
plot(sort(log(alphas)),1000*predict(fit.ie1,se=TRUE)$fit[order(alphas)],type='l',ylim=1000*c(-.1,.1),lty=2,lwd=2, axes=F, ann=F, cex.lab=1.2, col='blue')
abline(h=0,col='grey',lty=4)
axis(1, las=1, cex.axis=1.2)
axis(2, las=1, cex.axis=1.2)
box()
#Comm
fit.ie2=gam(ies[2,]~s(log(alphas),bs='cr',k=20),family=gaussian)
lines(sort(log(alphas)),1000*predict(fit.ie2,se=TRUE)$fit[order(alphas)],lty=1,lwd=2)
#lines(sort(log(alphas)),1000*c(predict(fit.ie2,se=TRUE)$fit-2*predict(fit.ie2,se=TRUE)$se.fit)[order(alphas)],lty=4)
#lines(sort(log(alphas)),1000*c(predict(fit.ie2,se=TRUE)$fit+2*predict(fit.ie2,se=TRUE)$se.fit)[order(alphas)],lty=4)
#Pool PE
fit.ie3=gam(ies[3,]~s(log(alphas),bs='cr',k=20),family=gaussian)
lines(sort(log(alphas)),1000*predict(fit.ie3,se=TRUE)$fit[order(alphas)],lty=3,lwd=2,col='blue')
#Curr K-M
fit.ie4=gam(ies[4,]~s(log(alphas),bs='cr',k=20),family=gaussian)
lines(sort(log(alphas)),1000*predict(fit.ie4,se=TRUE)$fit[order(alphas)],lty=2,lwd=2,col='red')
#Pool K-M
fit.ie5=gam(ies[5,]~s(log(alphas),bs='cr',k=20),family=gaussian)
lines(sort(log(alphas)),1000*predict(fit.ie5,se=TRUE)$fit[order(alphas)],lty=3,lwd=2,col='red')
title(ylab="1000xIME",xlab=expression(alpha), cex.lab=1.2)
legend('topright',legend=c('Comm PE','Current PE','Pooled PE','Current K-M','Pooled K-M'),lty=c(1,2,3,2,3), lwd=2, col=c('black','blue','blue','red','red'), cex=1.2, bty="n", seg.len = 3)

#############################
#         IMSE Plot         #
#############################
#Curr PE
fit.ise1=gam(log(ises[1,])~s(log(alphas),bs='cr',k=8),family=gaussian)
plot(sort(log(alphas)),1000*exp(predict(fit.ise1,se=TRUE)$fit)[order(alphas)],type='l',lty=2,lwd=2,ylim=1000*c(0,.005), axes=F, ann=F, cex.lab=1.2, col='blue')
axis(1, las=1, cex.axis=1.2)
axis(2, las=1, cex.axis=1.2)
box()
#Comm
fit.ise2=gam(log(ises[2,])~s(log(alphas),bs='cr',k=8),family=gaussian)
lines(sort(log(alphas)),1000*exp(predict(fit.ise2,se=TRUE)$fit)[order(alphas)],lty=1,lwd=2)
#lines(sort(log(alphas)),1000*c(predict(fit.ise2,se=TRUE)$fit-2*predict(fit.ise2,se=TRUE)$se.fit)[order(alphas)],lty=4)
#lines(sort(log(alphas)),1000*c(predict(fit.ise2,se=TRUE)$fit+2*predict(fit.ise2,se=TRUE)$se.fit)[order(alphas)],lty=4)
#Pool PE
fit.ise3=gam(log(ises[3,])~s(log(alphas),bs='cr',k=8),family=gaussian)
lines(sort(log(alphas)),1000*exp(predict(fit.ise3,se=TRUE)$fit)[order(alphas)],lty=3,lwd=2,col='blue')
#Curr K-M
fit.ise4=gam(log(ises[4,])~s(log(alphas),bs='cr',k=8),family=gaussian)
lines(sort(log(alphas)),1000*exp(predict(fit.ise4,se=TRUE)$fit)[order(alphas)],lty=2,lwd=2,col='red')
#Pool K-M
fit.ise5=gam(log(ises[5,])~s(log(alphas),bs='cr',k=8),family=gaussian)
lines(sort(log(alphas)),1000*exp(predict(fit.ise5,se=TRUE)$fit)[order(alphas)],lty=3,lwd=2,col='red')
title(ylab="1000xIMSE",xlab=expression(alpha), cex.lab=1.2)
legend('topright',legend=c('Comm PE','Current PE','Pooled PE','Current K-M','Pooled K-M'),lty=c(1,2,3,2,3), lwd=2, col=c('black','blue','blue','red','red'), cex=1.2, bty="n", seg.len = 3)

#############################
#       CI Width Plot       #
#############################
#Curr PE
fit.ciwd1=gam(ciwds[1,]~s(log(alphas),bs='cr',k=8),family=gaussian)
plot(sort(log(alphas)),predict(fit.ciwd1,se=TRUE)$fit[order(alphas)],type='l',lty=2,lwd=2,ylim=c(.11,.25), axes=F, ann=F, cex.lab=1.2,col='blue')
axis(1, las=1, cex.axis=1.2)
axis(2, las=1, cex.axis=1.2)
box()
#Comm
fit.ciwd2=gam(ciwds[2,]~s(log(alphas),bs='cr',k=8),family=gaussian)
lines(sort(log(alphas)),predict(fit.ciwd2,se=TRUE)$fit[order(alphas)],lty=1,lwd=2)
#lines(sort(log(alphas)),c(predict(fit.ciwd2,se=TRUE)$fit-2*predict(fit.ciwd2,se=TRUE)$se.fit)[order(alphas)],lty=4)
#lines(sort(log(alphas)),c(predict(fit.ciwd2,se=TRUE)$fit+2*predict(fit.ciwd2,se=TRUE)$se.fit)[order(alphas)],lty=4)
#Pool PE
fit.ciwd3=gam(ciwds[3,]~s(log(alphas),bs='cr',k=8),family=gaussian)
lines(sort(log(alphas)),predict(fit.ciwd3,se=TRUE)$fit[order(alphas)],lty=3,lwd=2,col='blue')
#Curr K-M
lalphas = log(alphas)
fit.ciwd4=gam(ciwds[4,]~s(lalphas,bs='cr',k=8),family=gaussian)
lines(sort(lalphas),predict(fit.ciwd4,newdata=data.frame(lalphas=log(alphas)),se=TRUE)$fit[order(lalphas)],lty=2,lwd=2,col='red')
#Pool K-M
fit.ciwd5=gam(ciwds[5,]~s(log(alphas),bs='cr',k=8),family=gaussian)
lines(sort(log(alphas)),predict(fit.ciwd5,se=TRUE)$fit[order(alphas)],lty=3,lwd=2,col='red')
title(ylab="95% Credible Interval Width for S(0.75)",xlab=expression(alpha), cex.lab=1.2)
legend('topright',legend=c('Comm PE','Current PE','Pooled PE','Current K-M','Pooled K-M'),lty=c(1,2,3,2,3), lwd=2, col=c('black','blue','blue','red','red'), cex=1.2, bty="n", seg.len = 3)

#############################
# Coverage Probability Plot #
#############################
#Curr PE
fit.cvr1=gam(cvrs[1,]~s(log(alphas),bs='cr',k=8),family=binomial)
plot(sort(log(alphas)),predict(fit.cvr1,type="response",se=TRUE)$fit[order(alphas)],type='l',lty=2,lwd=2,ylim=c(.5,1), axes=F, ann=F, cex.lab=1.2,col='blue')
abline(h=.95,lty=5,col='grey')
axis(1, las=1, cex.axis=1.2)
axis(2, las=1, cex.axis=1.2)
box()
#Comm
fit.cvr2=gam(cvrs[2,]~s(log(alphas),bs='cr',k=20),family=binomial)
lines(sort(log(alphas)),predict(fit.cvr2,type="response",se=TRUE)$fit[order(alphas)],lty=1,lwd=2)
#lines(sort(log(alphas)),c(predict(fit.cvr2,type="response",se=TRUE)$fit-2*predict(fit.cvr2,type="response",se=TRUE)$se.fit)[order(alphas)],lty=4)
#lines(sort(log(alphas)),c(predict(fit.cvr2,type="response",se=TRUE)$fit+2*predict(fit.cvr2,type="response",se=TRUE)$se.fit)[order(alphas)],lty=4)
#Pool PE
fit.cvr3=gam(cvrs[3,]~s(log(alphas),bs='cr',k=8),family=binomial)
lines(sort(log(alphas)),predict(fit.cvr3,type="response",se=TRUE)$fit[order(alphas)],lty=3,lwd=2,col='blue')
#Curr K-M
fit.cvr4=gam(cvrs[4,]~s(log(alphas),bs='cr',k=8),family=binomial)
lines(sort(log(alphas)),predict(fit.cvr4,type="response",se=TRUE)$fit[order(alphas)],lty=2,lwd=2,col='red')
#Pool K-M
fit.cvr5=gam(cvrs[5,]~s(log(alphas),bs='cr',k=8),family=binomial)
lines(sort(log(alphas)),predict(fit.cvr5,type="response",se=TRUE)$fit[order(alphas)],lty=3,lwd=2,col='red')
title(ylab="Coverage Probability of S(0.75)",xlab=expression(alpha), cex.lab=1.2)
legend('right',legend=c('Comm PE','Current PE','Pooled PE','Current K-M','Pooled K-M'),lty=c(1,2,3,2,3), lwd=2, col=c('black','blue','blue','red','red'), cex=1.2, bty="n", seg.len = 3)
# Restore default margins
par(mfrow=c(1,1),mar=c(5, 4, 4, 2) + 0.1)


